'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const http = require('http');
const EventEmitter = require('events');

const express = require('express');
const bodyParser = require('body-parser');

const marko = require('marko');
const markoExpress = require('marko/express');
const TestPage = marko.load(require('./pages/test-page'));

const { Server: WebSocketServer } = require('ws');

class Server extends EventEmitter {
  constructor({ outputDir, pageLasso, dependencies, testPage }) {
    super();

    const app = this._app = express();
    app.use(bodyParser.json());
    app.use(markoExpress());

    testPage = testPage && marko.load(testPage) || TestPage;

    app.get('/', (req, res) => {
      this.emit('start');

      res.marko(testPage, {
        lasso: pageLasso,
        dependencies
      });
    });

    app.post('/end-test', (req, res) => {
      const {
        errorMsg,
        testsPassed,
        coverageReport
      } = req.body;

      this.emit('end', {
        errorMsg,
        testsPassed,
        coverageReport
      });

      // return empty response as ack
      res.send({});
    });

    app.use('/static', express.static(outputDir));
  }

  getPort() {
    return this._port;
  }

  close() {
    if (this._server && this._server.listening) {
      this._server.close();
    }
  }

  listen() {
    var _this = this;

    return _asyncToGenerator(function* () {
      return new Promise(function (resolve) {
        const httpServer = _this._server = http.createServer(_this._app).listen(function () {
          const { port } = httpServer.address();
          _this._port = port;

          console.info(`Test server is listening on http://localhost:${port}...`);

          // webSockets are used to send stdout and console logs to server in order
          const webSocketServer = _this._webSocketServer = new WebSocketServer({
            server: httpServer,
            path: '/ws'
          });

          webSocketServer.on('connection', function (client) {
            client.on('message', function (rawMessage) {
              const { type, data } = JSON.parse(rawMessage);
              if (type === 'stdout') {
                _this.emit('stdout', data);
              } else {
                _this.emit('console', ...data);
              }
            });
            _this.emit('web-socket-connection', client);
          });

          resolve();
        });
      });
    })();
  }
}

module.exports = Server;