'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

require('marko/node-require').install();
require('lasso/node-require-no-op').enable('.less', '.css');
require('lasso/browser-refresh').enable('*.marko *.css *.less');

require('marko/browser-refresh').enable();

const puppeteer = require('puppeteer');

const assert = require('assert');
const EventEmitter = require('events');

const fs = require('fs');
const { promisify } = require('bluebird');

const writeFileAsync = promisify(fs.writeFile);
const mkdirAsync = promisify(fs.mkdir);

const uuid = require('uuid');
const rimrafAsync = promisify(require('rimraf'));

const Server = require('./Server');

const _prepareTestPageInput = require('./utils/prepareTestPageInput');

const DEFAULT_MOCHA_OPTIONS = {
  ui: 'bdd',
  reporter: 'spec',
  useColors: true
};

class TestRunner extends EventEmitter {
  constructor(options) {
    var _this;

    _this = super();

    assert(options, 'Test runner options must be provided (at least testFiles)');

    this._app = null;
    this._httpServer = null;
    this._webSocketServer = null;
    this._browser = null;
    this._page = null;

    const {
      testFiles,
      mochaOptions,
      lassoConfig,
      lassoDependencies,
      puppeteerLaunchOptions,
      puppeteerPageOptions,
      testPage,

      // test options
      _instrumentCode,
      _randomizeOutputDir
    } = options;

    this._mochaOptions = mochaOptions;
    this._puppeteerLaunchOptions = puppeteerLaunchOptions;
    this._puppeteerPageOptions = puppeteerPageOptions;

    assert(Array.isArray(testFiles), 'testFiles must be provided as an array');
    if (lassoDependencies) {
      assert(Array.isArray(lassoDependencies), 'lassoDependencies must be provided as an array');
    }

    // use output dir from lasso config if present
    const outputDir = lassoConfig && lassoConfig.outputDir || `./.mocha-puppeteer${_randomizeOutputDir ? uuid.v4() : ''}`;

    const instrumentCode = _instrumentCode === undefined ? !!process.env.NYC_CONFIG : !!_instrumentCode;

    const { pageLasso, dependencies } = _prepareTestPageInput({
      outputDir,
      instrumentCode,
      lassoConfig,
      lassoDependencies,
      testFiles
    });

    const server = this._server = new Server({
      outputDir,
      pageLasso,
      dependencies,
      testPage
    });

    server.on('console', console.log);
    server.on('stdout', output => process.stdout.write(output));

    server.on('start', () => this.emit('start'));

    server.on('end', (() => {
      var _ref = _asyncToGenerator(function* ({ errorMsg, testsPassed, coverageReport }) {
        server.close();
        _this._browser && _this._browser.close();

        if (coverageReport) {
          let ableToWriteFile = true;
          // write to nyc temp dir so that coverage can be collected
          try {
            yield mkdirAsync('./.nyc_output');
          } catch (err) {
            if (err.code !== 'EEXIST') {
              console.error('Unable to create temporary directory', err);
              ableToWriteFile = false;
            }
          }

          if (ableToWriteFile) {
            yield writeFileAsync('./.nyc_output/coverage.json', JSON.stringify(coverageReport));
          }
        }

        if (_randomizeOutputDir) {
          yield rimrafAsync(outputDir);
        }

        if (errorMsg) {
          _this.emit('error', new Error(errorMsg));
        } else {
          _this.emit('complete', { testsPassed });
        }
      });

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    })());
  }

  start() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      const server = _this2._server;
      yield server.listen();

      // config mocha options
      const mochaOptions = Object.assign({}, DEFAULT_MOCHA_OPTIONS, _this2._mochaOptions);

      const browser = _this2._browser = yield puppeteer.launch(_this2._puppeteerLaunchOptions);
      const page = _this2._page = yield browser.newPage();

      yield page.exposeFunction('puppeteerCommand', _this2.executePageCommand.bind(_this2));

      page.on('error', function (err) {
        return _this2.emit('error', err);
      });
      page.on('pageerror', function (err) {
        return _this2.emit('error', err);
      });

      const { columns } = process.stdout;

      // set viewport width and height to number of columns
      // for cleaner output
      page.setViewport({ width: columns, height: columns });

      yield page.goto(`http://localhost:${server.getPort()}` + `#${JSON.stringify({ mochaOptions })}`, _this2._puppeteerPageOptions);
    })();
  }

  executePageCommand(command) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      assert(command, 'Executing "puppeteerCommand" requires options');

      if (!_this3._page) {
        throw new Error('Cannot execute page command because the page is not ready');
      }

      const { type, args } = command;
      assert(type, '"type" is a required property for executing a "puppeteerCommand"');

      if (!_this3._page[type]) {
        throw new Error(`Error executing page command. Invalid puppeteer page command "${type}".`);
      }

      if (args) return _this3._page[type](...args);

      return _this3._page[type]();
    })();
  }
}

module.exports = TestRunner;